VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 1  'vbDataSource
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsOrderDetails"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' These property values can be displayed on frmDemo
Public Qty As Integer
Public UnitPrice As Currency
Public Discount As Single

' We will get above property values from rsGrid
Private rsGrid As ADODB.Recordset

'The value of mOrderID will determine OrderDetail Records
Private mOrderID As Long

' set up a temporary recordset so we can show edit changes in our frmDemo.DataGrid1
Private WithEvents rsTemp As ADODB.Recordset
Attribute rsTemp.VB_VarHelpID = -1

' notify frmDemo that values have changed and therefore update display
' not being used in this demo project - here for scalability
Public Event DetailsInfoChg()

Public Sub SortGrid(ByVal ColIndex As Integer)
' by sorting the underlying recordset we can sort our frmDemo.DataGrid1
' set temporary strings to comply with .Sort syntax
Dim SortAscStr As String, SortDescStr As String
SortAscStr = rsTemp.Fields(ColIndex).Name & " ASC"
SortDescStr = rsTemp.Fields(ColIndex).Name & " DESC"
' Logic: If it is not sorted ascending, sort ascending.
' If it is sorted ascending, sort descending
If rsTemp.Sort <> SortAscStr Then
    rsTemp.Sort = SortAscStr
Else
    rsTemp.Sort = SortDescStr
End If

End Sub


Property Let OrderID(ByVal lData As Long)

mOrderID = lData

' set our connection
Dim cn As ADODB.Connection
Set cn = New ADODB.Connection
cn.CursorLocation = adUseClient
cn.Open "PROVIDER=Microsoft.Jet.OLEDB.4.0;Data Source=" & _
    DB_PATH & ";"

' get our recordset
Set rsGrid = New ADODB.Recordset
Dim sql As String
sql = "SELECT OrderID, ProductID, Quantity, UnitPrice, Discount " & _
    "FROM [Order Details] " & _
    "WHERE [Order Details].OrderID = " & mOrderID

rsGrid.Open sql, cn, adOpenStatic, adLockOptimistic

' create a "mirror recordset so we can show user changes in our frmDemo.DataGrid1
Set rsTemp = CopyRecordset(rsGrid)
' move back to first record if there are records
If rsTemp.RecordCount > 0 Then
    rsTemp.MoveFirst
End If

End Property

Private Sub Class_GetDataMember(DataMember As String, Data As Object)
' This is a data source class - tell frmDemo.DataGrid1 which recordset in use
Set Data = rsTemp

End Sub

Private Sub rsTemp_FieldChangeComplete(ByVal cFields As Long, ByVal Fields As Variant, ByVal pError As ADODB.Error, adStatus As ADODB.EventStatusEnum, ByVal pRecordset As ADODB.Recordset)
' Let's update our properties and field values
With rsTemp
    Select Case True
        Case (Qty <> !Quantity)
            Qty = !Quantity
            !Total = Qty * !Net
        Case (UnitPrice <> !UnitPrice)
            UnitPrice = !UnitPrice
            !Net = UnitPrice - (UnitPrice * Discount)
            !Total = Qty * !Net
        Case (Discount <> !Discount)
            Discount = !Discount
            !Net = UnitPrice - (UnitPrice * Discount)
            !Total = Qty * !Net
    End Select
    ' tell frmDemo values have changed
    ' this event is not being used in this demo project
    RaiseEvent DetailsInfoChg
End With
End Sub

Private Sub rsTemp_MoveComplete(ByVal adReason As ADODB.EventReasonEnum, ByVal pError As ADODB.Error, adStatus As ADODB.EventStatusEnum, ByVal pRecordset As ADODB.Recordset)
' update our properties
With rsTemp
    If rsTemp.EOF Then
        Qty = 0
        UnitPrice = 0
        Discount = 0
    Else
        Qty = !Quantity
        UnitPrice = !UnitPrice
        Discount = !Discount
    End If
End With
RaiseEvent DetailsInfoChg
End Sub

' The following two functions are based on algorithms by Francesco Balena
' See pg. 681 "Programming MS VB 6.0"
' We will create a "mirror" recordset so our users can see the effects of
' cell editing in our frmDemo.DataGrid1

Private Function CopyRecordset(rs As ADODB.Recordset) As ADODB.Recordset
Dim newRS As New ADODB.Recordset, fld As ADODB.Field

Dim crUnitPrice As Currency, crNet As Currency
Dim iQuantity As Integer
Dim crDiscount As Single

Set newRS = CopyFields(rs)      ' copy the fields from rsGrid and add two calculated fields
' now that fields are copied, let's open our recordset
newRS.Open
If rs.RecordCount = 0 Then
    Set CopyRecordset = newRS
    Exit Function
End If
' loop through rs (points to rsGrid) and add values to newRS (points to rsTemp)
rs.MoveFirst
Do Until rs.EOF
    With newRS
        .AddNew
        !OrderID = rs!OrderID
        !ProductID = rs!ProductID
        ' Quantity
        iQuantity = rs!Quantity
        !Quantity = iQuantity
        ' UnitPrice
        crUnitPrice = rs!UnitPrice
        !UnitPrice = crUnitPrice
        ' Discount
        crDiscount = rs!Discount
        !Discount = crDiscount
        ' calculated fields -------------------------------
        ' Net
        crNet = crUnitPrice - (crUnitPrice * crDiscount)
        !Net = crNet
        ' Total
        !Total = iQuantity * crNet
        ' end calculated fields ---------------------------
    End With
    rs.MoveNext
Loop

Set CopyRecordset = newRS

End Function

Private Function CopyFields(rs As ADODB.Recordset) As ADODB.Recordset
' This function loops through the rs.Fields collection and makes identical fields
' in newRS (points to rsTemp) then adds two calculated fields Net, Total
Dim newRS As New ADODB.Recordset, fld As ADODB.Field
With newRS
    For Each fld In rs.Fields
        .Fields.Append fld.Name, fld.Type, fld.DefinedSize, fld.Attributes
    Next
    .Fields.Append "Net", adCurrency
    .Fields.Append "Total", adCurrency
End With

Set CopyFields = newRS

End Function

Private Sub Class_Terminate()
' Close Recordset
rsGrid.Close
End Sub




